
USE tempdb
GO

select * from sys.spatial_reference_systems;
go

CREATE FUNCTION dbo.measure_globe(@srid INT)
RETURNS FLOAT
AS
BEGIN
DECLARE @g geography = geography::STGeomFromText('FULLGLOBE',@srid);
RETURN @g.STArea()
END 
GO

CREATE FUNCTION dbo.get_ellipsoid(@wkt NVARCHAR(4000))
RETURNS NVARCHAR(4000)
AS
BEGIN
-- grungy string-handling alert
DECLARE @begin INT = PATINDEX('%ELLIPSOID%',@wkt) + 11 
DECLARE @rest NVARCHAR(4000) = SUBSTRING(@wkt, @begin, 4000)
DECLARE @end INT = CHARINDEX('"', @rest)
RETURN SUBSTRING(@rest, 1, @end-1)
END
GO 

SELECT spatial_reference_id AS SRID, 
       dbo.measure_globe(spatial_reference_id) AS Area,
       unit_of_measure,
       well_known_text,
       dbo.get_ellipsoid(well_known_text) AS Ellipsoid
FROM sys.spatial_reference_systems
ORDER BY area, ellipsoid, spatial_reference_id  
GO

-- Spatial Union Aggregate - Find Percentage of Earth's Surface that is water
DECLARE @fg GEOGRAPHY = GEOGRAPHY::STGeomFromText('FULLGLOBE',4326);
DECLARE @dlm GEOGRAPHY = (SELECT GEOGRAPHY::UnionAggregate(geog) from SpatialSamples..cntry00)
DECLARE @water GEOGRAPHY = @fg.STDifference(@dlm)

SELECT @water.STArea() as [TotalWaterAreaInSquareMeters]
--Results: 376038730061515 meters squared

SELECT (@water.STArea() / @fg.STArea()) * 100 [PercentOfEarthCoveredByWater]
--Results: 73.7235983088032 %
GO




