
USE tempdb;
GO

-- 1. FULLGLOBE spatial type
DECLARE @fg GEOGRAPHY = GEOGRAPHY::Parse('FULLGLOBE')
SELECT @fg.STArea()
GO

DECLARE @fg GEOGRAPHY = GEOGRAPHY::STGeomFromText('FULLGLOBE',4326)
SELECT @fg.STArea()
GO

-- 2. ReorientObject - interchange interior and exterior region
DECLARE @point GEOGRAPHY = GEOGRAPHY::Point(47.5,-122.33,4326);
DECLARE @area1 FLOAT = @point.STBuffer(1000).STArea();
DECLARE @area2 FLOAT = @point.STBuffer(1000).ReorientObject().STArea();
DECLARE @surface_area FLOAT = (@area1 + @area2) / (1000 * 1000)

SELECT @surface_area AS SURFACE_AREA_KM_SQ
SELECT @area1 as SMALL_AREA_Square_Meters
SELECT @area2 as LARGE_AREA_KM

--Published value for WGS84 Equatorial Circumference: 40,075.017 km
--Equatorial Circumference defined with 5 points
DECLARE @eq_cir GEOGRAPHY = GEOGRAPHY::STGeomFromText('LineString (0 0,-100 0, -179 0, 101 0, 0 0)',4326)
SELECT @eq_cir.STLength()
GO
--Results: 40,075,016.6627362 meters

--Equatorial Circumference defined with 3 points:
DECLARE @eq_cir GEOGRAPHY = GEOGRAPHY::STGeomFromText('LineString (0 0, -179 0, 0 0)',4326)
SELECT @eq_cir.STIsValid()
--Results: 0 Not valid (line doubles back on itself)

-- 3. STLength works even on invalid linestrings
SELECT @eq_cir.STLength()
GO
--Results: 39,852,377.4847659

--Published value for WGS84 Polar Circumference: 40 007.863 km km
--Equatorial Circumference defined with 5 points
DECLARE @eq_cir GEOGRAPHY = GEOGRAPHY::STGeomFromText('LineString (0 0,0 90, 180 80, 180 -70, 0 -45, 0 0)',4326)
SELECT @eq_cir.STIsValid()
SELECT @eq_cir.STLength()
GO
--Results: 40,007,862.8298599 meters

--
-- CALCULATE APPROXIMATE AREA WATER COVERING THE GLOBE (Fresh and Salt Water)
--
DECLARE @fg GEOGRAPHY = GEOGRAPHY::STGeomFromText('FULLGLOBE',4326);
-- 4. Spatial Union Aggregate
DECLARE @dlm GEOGRAPHY = (SELECT GEOGRAPHY::UnionAggregate(geog) from SpatialSamples..cntry00)
DECLARE @water GEOGRAPHY = @fg.STDifference(@dlm)
SELECT @water.STArea()
--Results: 376038730061515 meters squared
SELECT (@water.STArea() / @fg.STArea()) * 100
--Results: 73.7235983088032 %
GO

-- 5. CIRCULARSTRING, COMPOUNDCURVE, CURVEPOLYGON spatial types
-- geometric semicircle
DECLARE @g geometry;
SET @g = geometry::Parse('COMPOUNDCURVE(CIRCULARSTRING(0 2, 2 0, 4 2), (4 2, 0 2))');
SELECT @g.STLength();
select @g
select @g.STCurveToLine()

-- geometric full circle 
DECLARE @g2 geometry;
SET @g2 = geometry::Parse('COMPOUNDCURVE(CIRCULARSTRING(0 2, 2 0, 4 2), CIRCULARSTRING(4 2, 2 4, 0 2))');
SELECT 'Circle Two', @g2.STLength() AS Perimeter; -- now we get an accurate amount
select @g2
select @g2.STCurveToLine()  -- required for visibility in spatial resuls tab prior to RTM

DECLARE @p GEOGRAPHY = GEOGRAPHY::Point(47.5,-122.33,4326)
SELECT @p.BufferWithCurves(1000)
SELECT (GEOGRAPHY::Point(47.5,-122.33,4326)).BufferWithCurves(1000).STAsText()
--select @p.STCurveToLine() -- this confuses spatial results tab

-- Get earth's surface area
DECLARE @P GEOGRAPHY = GEOGRAPHY::STGeomFromText('CURVEPOLYGON 
  (CIRCULARSTRING 
    (  0 0, 
     -90 0, 
     180 0, 
      90 0, 
       0 0))', 4326);
--SELECT @p.STIsValid();  -- Returns: 1 (valid)
SELECT @p.STArea();  -- area: 255032815307904
SELECT @P.STArea() * 2 AS WGS84_SURFACE_AREA -- 510065630615808 square meters
go

-- BufferWithCurves around a point produces a circle
DECLARE @g geometry= 'POINT(3 4)';
SELECT @g.BufferWithCurves(2).ToString();
SELECT @g.BufferWithCurves(2)
GO

-- BufferWithCurves produces less points than STBuffer
DECLARE @g geometry= 'LINESTRING(3 4, 8 11)';
SELECT @g.BufferWithCurves(2).ToString();
SELECT @g.BufferWithCurves(2)
SELECT @g.STBuffer(2)

SELECT @g.STBuffer(2).STNumPoints()
SELECT @g.BufferWithCurves(2).STNumPoints()



