
USE AdventureWorks2008R2
GO

CREATE SPATIAL INDEX Index_geography_HHHH_Address
   ON Person.Address(SpatialLocation)
   USING GEOGRAPHY_GRID
   WITH (
	GRIDS = (HIGH, HIGH, HIGH, HIGH),
    CELLS_PER_OBJECT = 64,
    PAD_INDEX  = ON );

SET Statistics IO ON
GO

CHECKPOINT
DBCC DROPCLEANBUFFERS
DBCC FREEPROCCACHE

select * from sys.dm_exec_query_stats
cross apply sys.dm_exec_sql_text(plan_handle)

-- doesn't use index in 2008
-- uses index in Denali, cost 14.7
-- Table 'Worktable'. Scan count 3, logical reads 175, physical reads 0, read-ahead reads 0, lob logical reads 80, lob physical reads 0, lob read-ahead reads 0.
-- Table 'Address'. Scan count 0, logical reads 122, physical reads 1, read-ahead reads 16, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
-- Table 'extended_index_373576369_384001'. Scan count 3181, logical reads 6836, physical reads 1, read-ahead reads 32, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

DECLARE @g geography = 'POINT(-121.626 47.8315)';
SELECT TOP(7) SpatialLocation.ToString(), City 
FROM Person.Address
WHERE SpatialLocation.STDistance(@g) IS NOT NULL
ORDER BY SpatialLocation.STDistance(@g)

-- Error if hinted
-- Denali - you can hint it
DECLARE @g geography = 'POINT(-121.626 47.8315)';
SELECT TOP(7) SpatialLocation.ToString(), City 
FROM Person.Address with (index=Index_geography_auto_Address)
WHERE SpatialLocation.STDistance(@g) IS NOT NULL
ORDER BY SpatialLocation.STDistance(@g)
GO

-- Use distance limit
-- Plan cost 4.06683
-- Table 'Worktable'. Scan count 0, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
-- Table 'Address'. Scan count 0, logical reads 4406, physical reads 2, read-ahead reads 320, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
-- Table 'extended_index_373576369_384001'. Scan count 1988, logical reads 4250, physical reads 1, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

DECLARE @g geography = 'POINT(-121.626 47.8315)';
SELECT TOP(7) SpatialLocation.ToString(), City 
FROM Person.Address
WHERE SpatialLocation.STDistance(@g) < 100000
ORDER BY SpatialLocation.STDistance(@g)

-- Using sp_executesql
DECLARE @g geography = 'POINT(-121.626 47.8315)';
EXEC sp_executesql 
N'SELECT TOP(7) SpatialLocation.ToString(), City 
FROM Person.Address
WHERE SpatialLocation.STDistance(@g1) < 100000
ORDER BY SpatialLocation.STDistance(@g1)', N'@g1 geography', @g1=@g

-- use sproc
CREATE PROC Top7LimitDistanceB (@g1 geography)
AS
SELECT TOP(7) SpatialLocation.ToString(), City, SpatialLocation.STDistance(@g1) 
FROM Person.Address
WHERE SpatialLocation.STDistance(@g1) < 100000
ORDER BY SpatialLocation.STDistance(@g1)

DECLARE @g geography = 'POINT(-121.626 47.8315)';
EXEC Top7LimitDistanceB @g
GO

-- no index used here
DECLARE @g geography = 'POINT(-121.626 47.8315)';
EXEC sp_executesql 
N'SELECT TOP(7) SpatialLocation.ToString(), City, SpatialLocation.STDistance(@g)/1000 
FROM Person.Address
WHERE SpatialLocation.STDistance(@g) IS NOT NULL
ORDER BY SpatialLocation.STDistance(@g) OPTION(MAXDOP 1)', N'@g geography', @g

-- index used
-- Table 'Worktable'. Scan count 3, logical reads 175, physical reads 0, read-ahead reads 0, lob logical reads 80, lob physical reads 0, lob read-ahead reads 0.
-- Table 'Address'. Scan count 0, logical reads 91, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
-- Table 'extended_index_373576369_384001'. Scan count 3181, logical reads 6776, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
DECLARE @g geography = 'POINT(-121.626 47.8315)';
EXEC sp_executesql 
N'SELECT TOP(7) SpatialLocation.ToString(), City
FROM Person.Address
WHERE SpatialLocation.STDistance(@g1) IS NOT NULL
ORDER BY SpatialLocation.STDistance(@g1) OPTION(MAXDOP 1)', N'@g1 geography', @g1=@g

-- cannot use spatial index (needs where clause)
USE AdventureWorks2008R2
GO

DECLARE @g geography = 'POINT(-121.626 47.8315)';
SELECT TOP(7) SpatialLocation.ToString(), City FROM Person.Address
ORDER BY SpatialLocation.STDistance(@g);

-- Isaac nearest neighbor

SELECT TOP 100000 IDENTITY(int,1,1) AS n 
INTO dbo.numbers 
FROM MASTER..spt_values a, MASTER..spt_values b
 
CREATE UNIQUE CLUSTERED INDEX idx_1 ON numbers(n)

select * from dbo.numbers

-- cost 0.517
-- Table 'Worktable'. Scan count 0, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
-- Table 'Address'. Scan count 0, logical reads 18, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
-- Table 'extended_index_373576369_384001'. Scan count 1548, logical reads 3306, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
-- Table 'numbers'. Scan count 1, logical reads 2, physical reads 2, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
DECLARE @start FLOAT = 1000; 
DECLARE @g geography = 'POINT(-121.626 47.8315)'; 
WITH NearestPoints AS
(
   SELECT TOP(7) SpatialLocation.ToString() loc, City, SpatialLocation.STDistance(@g) as dist, n
   FROM dbo.numbers 
   JOIN Person.Address T with (index=Index_geography_HHHH_Address) 
   ON T.SpatialLocation.STDistance(@g) < @start*POWER(2,dbo.numbers.n)
   ORDER BY dbo.numbers.n
)
SELECT TOP(7) * FROM NearestPoints
ORDER BY n, dist